@@ -13,7 +13,7 @@ import pytest
 
 import blosc2
 from blosc2.lazyexpr import ne_evaluate
-from blosc2.ndarray import get_chunks_idx
+from blosc2.ndarray import get_chunks_idx, npvecdot
 
 NITEMS_SMALL = 1_000
 NITEMS = 10_000
@@ -365,7 +365,7 @@ def test_functions(function, dtype_fixture, shape_fixture):
         # ("scalar", "scalar") # Not supported by LazyExpr
     ],
 )
-def test_arctan2_pow(urlpath, shape_fixture, dtype_fixture, function, value1, value2):  # noqa: C901
+def test_arctan2_pow(urlpath, shape_fixture, dtype_fixture, function, value1, value2):
     nelems = np.prod(shape_fixture)
     if urlpath is None:
         urlpath1 = urlpath2 = urlpath_save = None
@@ -1088,7 +1088,7 @@ def test_eval_getitem2():
     np.testing.assert_allclose(expr[0], nres[0])
     np.testing.assert_allclose(expr[1:, :7], nres[1:, :7])
     np.testing.assert_allclose(expr[0:10:2], nres[0:10:2])
-    # This works, but it is not very efficient since it relies on blosc2.ndarray.slice for non-unit steps
+    # Now relies on inefficient blosc2.ndarray.slice for non-unit steps but only per chunk (not for whole result)
     np.testing.assert_allclose(expr.slice((slice(None, None, None), slice(0, 10, 2)))[:], nres[:, 0:10:2])
 
     # Small test for broadcasting
@@ -1097,7 +1097,7 @@ def test_eval_getitem2():
     np.testing.assert_allclose(expr[0], nres[0])
     np.testing.assert_allclose(expr[1:, :7], nres[1:, :7])
     np.testing.assert_allclose(expr[:, 0:10:2], nres[:, 0:10:2])
-    # This works, but it is not very efficient since it relies on blosc2.ndarray.slice for non-unit steps
+    # Now relies on inefficient blosc2.ndarray.slice for non-unit steps but only per chunk (not for whole result)
     np.testing.assert_allclose(expr.slice((slice(None, None, None), slice(0, 10, 2)))[:], nres[:, 0:10:2])
 
 
@@ -1105,13 +1105,13 @@ def test_eval_getitem2():
 def test_eval_slice(array_fixture):
     a1, a2, a3, a4, na1, na2, na3, na4 = array_fixture
     expr = blosc2.lazyexpr("a1 + a2 - (a3 * a4)", operands={"a1": a1, "a2": a2, "a3": a3, "a4": a4})
-    nres = ne_evaluate("na1 + na2 - (na3 * na4)")[:2]
-    res = expr.slice(slice(0, 2))
+    nres = ne_evaluate("na1 + na2 - (na3 * na4)")
+    res = expr.slice(slice(0, 8, 2))
     assert isinstance(res, blosc2.ndarray.NDArray)
-    np.testing.assert_allclose(res[:], nres)
-    res = expr[:2]
+    np.testing.assert_allclose(res[:], nres[:8:2])
+    res = expr[:8:2]
     assert isinstance(res, np.ndarray)
-    np.testing.assert_allclose(res, nres)
+    np.testing.assert_allclose(res, nres[:8:2])
 
     # string lazy expressions automatically use .slice internally
     expr1 = blosc2.lazyexpr("a1 * a2", operands={"a1": a1, "a2": a2})
@@ -1123,6 +1123,18 @@ def test_eval_slice(array_fixture):
     np.testing.assert_allclose(res[()], nres)
 
 
+def test_rebasing(array_fixture):
+    a1, a2, a3, a4, na1, na2, na3, na4 = array_fixture
+    expr = blosc2.lazyexpr("a1 + a2 - (a3 * a4)", operands={"a1": a1, "a2": a2, "a3": a3, "a4": a4})
+    assert expr.expression == "(o0 + o1 - o2 * o3)"
+
+    expr = blosc2.lazyexpr("a1")
+    assert expr.expression == "(o0)"
+
+    expr = blosc2.lazyexpr("a1[:10]")
+    assert expr.expression == "(o0.slice((slice(None, 10, None),)))"
+
+
 # Test get_chunk method
 @pytest.mark.heavy
 def test_get_chunk(array_fixture):
@@ -1340,7 +1352,7 @@ def test_sort():
     ],
 )
 @pytest.mark.parametrize("getitem", [True, False])
-@pytest.mark.parametrize("item", [None, slice(10), slice(0, 10, 2)])
+@pytest.mark.parametrize("item", [(), slice(10), slice(0, 10, 2)])
 def test_only_ndarrays_or_constructors(obj, getitem, item):
     arr = blosc2.arange(10)  # is a test case
     larr = blosc2.lazyexpr(obj)
@@ -1353,7 +1365,7 @@ def test_only_ndarrays_or_constructors(obj, getitem, item):
     else:
         b = larr.compute(item)
         assert isinstance(b, blosc2.NDArray)
-    if item is None:
+    if item == ():
         assert b.shape == larr.shape
     assert b.dtype == larr.dtype
     if not isinstance(obj, str):
@@ -1363,11 +1375,13 @@ def test_only_ndarrays_or_constructors(obj, getitem, item):
 @pytest.mark.parametrize("func", ["cumsum", "cumulative_sum", "cumprod"])
 def test_numpy_funcs(array_fixture, func):
     a1, a2, a3, a4, na1, na2, na3, na4 = array_fixture
-    npfunc = getattr(np, func)
-    d_blosc2 = npfunc(((a1**3 + blosc2.sin(na2 * 2)) < a3) & (na2 > 0), axis=0)
-    npfunc = getattr(np, func)
-    d_numpy = npfunc(((na1**3 + np.sin(na2 * 2)) < na3) & (na2 > 0), axis=0)
-    np.testing.assert_equal(d_blosc2, d_numpy)
+    try:
+        npfunc = getattr(np, func)
+        d_blosc2 = npfunc(((a1**3 + blosc2.sin(na2 * 2)) < a3) & (na2 > 0), axis=0)
+        d_numpy = npfunc(((na1**3 + np.sin(na2 * 2)) < na3) & (na2 > 0), axis=0)
+        np.testing.assert_equal(d_blosc2, d_numpy)
+    except AttributeError:
+        pytest.skip("NumPy version has no cumulative_sum function.")
 
 
 # Test the LazyExpr when some operands are missing (e.g. removed file)
@@ -1426,6 +1440,20 @@ def test_chain_expressions():
     res = expr_final.compute()
     np.testing.assert_allclose(res[:], nres)
 
+    # Test that update_expr does not alter expr1
+    expr1 = "a + b"
+    expr2 = "sin(a) + tan(c)"
+    lexpr1 = blosc2.lazyexpr(expr1)
+    lexpr2 = blosc2.lazyexpr(expr2)
+    lexpr3 = lexpr1 + lexpr2
+    assert lexpr1.expression == lexpr1.expression
+    assert lexpr1.operands == lexpr1.operands
+    assert lexpr2.expression == lexpr2.expression
+    assert lexpr2.operands == lexpr2.operands
+    lexpr1 += lexpr2
+    assert lexpr1.expression == lexpr3.expression
+    assert lexpr1.operands == lexpr3.operands
+
 
 # Test the chaining of multiple persistent lazy expressions
 def test_chain_persistentexpressions():
@@ -1437,7 +1465,7 @@ def test_chain_persistentexpressions():
 
     le1 = a**3 + blosc2.sin(a**2)
     le2 = le1 < c
-    le3 = b < 0
+    le3 = le2 & (b < 0)
     le4 = le2 & le3
 
     le1_ = blosc2.lazyexpr("a ** 3 + sin(a ** 2)", {"a": a})
@@ -1457,6 +1485,10 @@ def test_chain_persistentexpressions():
     myle4 = blosc2.open("expr4.b2nd")
     assert (myle4[:] == le4[:]).all()
 
+    # Remove files
+    for f in ["expr1.b2nd", "expr2.b2nd", "expr3.b2nd", "expr4.b2nd", "a.b2nd", "b.b2nd", "c.b2nd"]:
+        blosc2.remove_urlpath(f)
+
 
 @pytest.mark.parametrize(
     "values",
@@ -1473,8 +1505,8 @@ def test_chain_persistentexpressions():
 def test_scalar_dtypes(values):
     value1, value2 = values
     dtype1 = (value1 + value2).dtype
-    avalue1 = blosc2.asarray(value1) if hasattr(value1, "shape") else value1
-    avalue2 = blosc2.asarray(value2) if hasattr(value2, "shape") else value2
+    avalue1 = blosc2.asarray(value1) if not np.isscalar(value1) else value1
+    avalue2 = blosc2.asarray(value2) if not np.isscalar(value2) else value2
     dtype2 = (avalue1 * avalue2).dtype
     assert dtype1 == dtype2, f"Expected {dtype1} but got {dtype2}"
 
@@ -1490,3 +1522,249 @@ def test_to_cframe():
     assert arr.shape == expr.shape
     assert arr.dtype == expr.dtype
     assert np.allclose(arr[:], expr[:])
+
+
+# Test for the bug where multiplying two complex lazy expressions would fail with:
+# ValueError: invalid literal for int() with base 10: '0,'
+def test_complex_lazy_expression_multiplication():
+    # Create test data similar to the animated plot scenario
+    width, height = 64, 64
+    x = np.linspace(-4 * np.pi, 4 * np.pi, width)
+    y = np.linspace(-4 * np.pi, 4 * np.pi, height)
+    X, Y = np.meshgrid(x, y)
+
+    # Convert to blosc2 arrays
+    X_b2 = blosc2.asarray(X)
+    Y_b2 = blosc2.asarray(Y)
+
+    # Create the complex expressions that were causing the bug
+    time_factor = 0.5
+
+    # First complex expression: R * 4 - time_factor * 2
+    R = np.sqrt(X_b2**2 + Y_b2**2)
+    expr1 = R * 4 - time_factor * 2
+
+    # Second complex expression: theta * 6
+    theta = np.arctan2(Y_b2, X_b2)
+    expr2 = theta * 6
+
+    # Apply functions to create more complex expressions
+    sin_expr = np.sin(expr1)
+    cos_expr = np.cos(expr2)
+
+    # This multiplication was failing before the fix
+    result_expr = sin_expr * cos_expr
+
+    # Evaluate the expression - this should not raise an error
+    result = result_expr.compute()
+
+    # Verify the result matches numpy computation using the same approach
+    # Use the blosc2 arrays converted to numpy to ensure consistency
+    R_np = np.sqrt(X_b2[:] ** 2 + Y_b2[:] ** 2)
+    theta_np = np.arctan2(Y_b2[:], X_b2[:])
+    expected = np.sin(R_np * 4 - time_factor * 2) * np.cos(theta_np * 6)
+
+    np.testing.assert_allclose(result, expected, rtol=1e-14, atol=1e-14)
+
+    # Also test getitem access
+    np.testing.assert_allclose(result_expr[:], expected, rtol=1e-14, atol=1e-14)
+
+
+# Test checking that objects following the blosc2.Array protocol can be operated with
+def test_minimal_protocol():
+    class NewObj:
+        def __init__(self, a):
+            self.a = a
+
+        @property
+        def shape(self):
+            return self.a.shape
+
+        @property
+        def dtype(self):
+            return self.a.dtype
+
+        def __getitem__(self, key):
+            return self.a[key]
+
+        def __len__(self):
+            return len(self.a)
+
+    a = np.arange(100, dtype=np.int64).reshape(10, 10)
+    b = NewObj(a)
+    c = blosc2.asarray(a)
+    lb = blosc2.lazyexpr("b + c + 1")
+
+    np.testing.assert_array_equal(lb[:], a + a + 1)
+
+
+def test_not_numexpr():
+    shape = (20, 20)
+    a = blosc2.linspace(0, 20, num=np.prod(shape), shape=shape)
+    b = blosc2.ones((20, 1))
+    d_blosc2 = blosc2.evaluate("logaddexp(a, b) + a")
+    npa = a[()]
+    npb = b[()]
+    np.testing.assert_array_almost_equal(d_blosc2, np.logaddexp(npa, npb) + npa)
+    # TODO: Implement __add__ etc. for LazyUDF so this line works
+    # d_blosc2 = blosc2.evaluate(f"logaddexp(a, b) + clip(a, 6, 12)")
+    arr = blosc2.lazyexpr("matmul(a, b)")
+    assert isinstance(arr, blosc2.LazyExpr)
+    np.testing.assert_array_almost_equal(arr[()], np.matmul(npa, npb))
+
+
+def test_lazylinalg():
+    """
+    Test the shape parser for linear algebra funcs
+    """
+    # --- define base shapes ---
+    shapes = {
+        "A": (3, 4),
+        "B": (4, 5),
+        "C": (2, 3, 4),
+        "D": (1, 5, 1),
+        "x": (10,),
+        "y": (10,),
+    }
+    s = shapes["x"]
+    x = blosc2.linspace(0, np.prod(s), shape=s)
+    s = shapes["y"]
+    y = blosc2.linspace(0, np.prod(s), shape=s)
+    s = shapes["A"]
+    A = blosc2.linspace(0, np.prod(s), shape=s)
+    s = shapes["B"]
+    B = blosc2.linspace(0, np.prod(s), shape=s)
+    s = shapes["C"]
+    C = blosc2.linspace(0, np.prod(s), shape=s)
+    s = shapes["D"]
+    D = blosc2.linspace(0, np.prod(s), shape=s)
+
+    npx = x[()]
+    npy = y[()]
+    npA = A[()]
+    npB = B[()]
+    npC = C[()]
+    npD = D[()]
+
+    # --- concat ---
+    out = blosc2.lazyexpr("concat((x, y), axis=0)")
+    npres = np.concatenate((npx, npy), axis=0)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- diagonal ---
+    out = blosc2.lazyexpr("diagonal(A)")
+    npres = np.diagonal(npA)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- expand_dims ---
+    out = blosc2.lazyexpr("expand_dims(x, axis=0)")
+    npres = np.expand_dims(npx, axis=0)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- matmul ---
+    out = blosc2.lazyexpr("matmul(A, B)")
+    npres = np.matmul(npA, npB)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- matrix_transpose ---
+    out = blosc2.lazyexpr("matrix_transpose(A)")
+    npres = np.matrix_transpose(npA) if np.__version__.startswith("2.") else npA.T
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+    out = blosc2.lazyexpr("C.mT")
+    npres = C.mT
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+    out = blosc2.lazyexpr("A.T")
+    npres = npA.T
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- outer ---
+    out = blosc2.lazyexpr("outer(x, y)")
+    npres = np.outer(npx, npy)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- permute_dims ---
+    out = blosc2.lazyexpr("permute_dims(C, axes=(2,0,1))")
+    npres = np.transpose(npC, axes=(2, 0, 1))
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- squeeze ---
+    out = blosc2.lazyexpr("squeeze(D)")
+    npres = np.squeeze(npD)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    out = blosc2.lazyexpr("D.squeeze()")
+    npres = np.squeeze(npD)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- stack ---
+    out = blosc2.lazyexpr("stack((x, y), axis=0)")
+    npres = np.stack((npx, npy), axis=0)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- tensordot ---
+    out = blosc2.lazyexpr("tensordot(A, B, axes=1)")
+    npres = np.tensordot(npA, npB, axes=1)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- vecdot ---
+    out = blosc2.lazyexpr("vecdot(x, y)")
+    npres = npvecdot(npx, npy)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+    # --- batched matmul ---
+    shapes = {
+        "A": (1, 3, 4),
+        "B": (3, 4, 5),
+    }
+    s = shapes["A"]
+    A = blosc2.linspace(0, np.prod(s), shape=s)
+    npA = A[()]  # actual numpy array
+    s = shapes["B"]
+    B = blosc2.linspace(0, np.prod(s), shape=s)
+    npB = B[()]  # actual numpy array
+
+    out = blosc2.lazyexpr("matmul(A, B)")
+    npres = np.matmul(npA, npB)
+    assert out.shape == npres.shape
+    np.testing.assert_array_almost_equal(out[()], npres)
+
+
+# Test for issue #503 (LazyArray.compute() should honor out param)
+def test_lazyexpr_compute_out():
+    # check reductions
+    a = blosc2.ones(10)
+    out = blosc2.zeros(1)
+    lexpr = blosc2.lazyexpr("sum(a)")
+    assert lexpr.compute(out=out) is out
+    assert out[0] == 10
+    assert lexpr.compute() is not out
+
+    # check normal expression
+    a = blosc2.ones(10)
+    out = blosc2.zeros(10)
+    lexpr = blosc2.lazyexpr("sin(a)")
+    assert lexpr.compute(out=out) is out
+    assert out[0] == np.sin(1)
+    assert lexpr.compute() is not out
+
+
+def test_lazyexpr_2args():
+    a = blosc2.ones(10)
+    lexpr = blosc2.lazyexpr("sin(a)")
+    newexpr = blosc2.hypot(lexpr, 3)
+    assert newexpr.expression == "hypot((sin(o0)), 3)"
+    assert newexpr.operands["o0"] is a
