@@ -57,6 +57,8 @@ def test_ndarray_cframe(contiguous, urlpath, cparams, dparams, nchunks, copy):
         ((200, 10), 2),
         ((200, 10, 10), 2),
         ((200, 10, 10), 40),
+        ((200, 10, 10), -1),
+        ((200, 10, 10), -3),
         ((200, 10, 10, 10), 9),
     ],
 )
@@ -292,31 +294,38 @@ def test_oindex():
     np.testing.assert_allclose(arr[:], nparr)
 
 
-def test_findex():
-    # Test 1d fast path
+@pytest.mark.parametrize("c", [None, 10])
+def test_fancy_index(c):
+    # Test 1d
     ndim = 1
-    d = 1 + int(blosc2.MAX_FAST_PATH_SIZE / 8)  # just over fast path size
+    chunks = (c,) * ndim if c is not None else None
+    dtype = np.dtype("float")
+    d = 1 + int(1000 / dtype.itemsize) if c is None else 50
     shape = (d,) * ndim
-    arr = blosc2.linspace(0, 100, num=np.prod(shape), shape=shape, dtype=np.float64)
+    arr = blosc2.linspace(0, 100, num=np.prod(shape), shape=shape, dtype=dtype, chunks=chunks)
     rng = np.random.default_rng()
     idx = rng.integers(low=0, high=d, size=(d // 4,))
     nparr = arr[:]
     b = arr[idx]
     n = nparr[idx]
     np.testing.assert_allclose(b, n)
+    b = arr[[[idx[::-1]], [idx]]]
+    n = nparr[[[idx[::-1]], [idx]]]
+    np.testing.assert_allclose(b, n)
 
     ndim = 3
-    d = 1 + int((blosc2.MAX_FAST_PATH_SIZE / 8) ** (1 / ndim))  # just over fast path size
+    d = 1 + int((1000 / 8) ** (1 / ndim)) if c is None else d  # just over numpy fast path size
     shape = (d,) * ndim
-    arr = blosc2.linspace(0, 100, num=np.prod(shape), shape=shape, dtype=np.float64)
+    chunks = (c,) * ndim if c is not None else None
+    arr = blosc2.linspace(0, 100, num=np.prod(shape), shape=shape, dtype=dtype, chunks=chunks)
     rng = np.random.default_rng()
-    idx = rng.integers(low=0, high=d, size=(100,))
+    idx = rng.integers(low=-d, high=d, size=(100,))  # mix of +ve and -ve indices
 
     row = idx
     col = rng.permutation(idx)
     mask = rng.integers(low=0, high=2, size=(d,)) == 1
 
-    ## Test fancy indexing for different use cases
+    # Test fancy indexing for different use cases
     m, M = np.min(idx), np.max(idx)
     nparr = arr[:]
     # i)
@@ -347,6 +356,15 @@ def test_findex():
     b = arr[row[:, None], mask]
     n = nparr[row[:, None], mask]
     np.testing.assert_allclose(b, n)
+
+    # indices and negative slice steps
+    b = arr[row, d // 2 :: -1]
+    n = nparr[row, d // 2 :: -1]
+    np.testing.assert_allclose(b, n)
+    b = arr[M // 2 :: -4, row, d // 2 :: -3]
+    n = nparr[M // 2 :: -4, row, d // 2 :: -3]
+    np.testing.assert_allclose(b, n)
+
     # Transposition test (3rd example is transposed)
     b1 = arr[:, [0, 1], 0]
     b2 = arr[[0, 1], 0, :]
@@ -354,7 +372,98 @@ def test_findex():
     n2 = nparr[[0, 1], 0, :]
     np.testing.assert_allclose(b1, n1)
     np.testing.assert_allclose(b2, n2)
-    # TODO: Support array indices separate by slices
+    # TODO: Support array indices separated by slices
     # b3 = arr[0, :, [0, 1]]
     # n3 = nparr[0, :, [0, 1]]
     # np.testing.assert_allclose(b3, n3)
+
+
+@pytest.mark.parametrize(
+    "arr",
+    [
+        np.random.default_rng().random((2, 1000, 10, 8, 3)).astype(np.float32),
+        blosc2.asarray(np.random.default_rng().random((2, 1000, 10, 8, 3)).astype(np.float32)),
+    ],
+)
+def test_strided_output(arr):
+    def fancy_strided_output(inputs, output_indices, stride=1):
+        b, t, *f = inputs.shape
+        oi = np.asarray(output_indices, dtype=np.int32)
+
+        start = np.amax(output_indices)
+        win_starts = np.arange(start, t, stride, dtype=np.int32)
+        rel_idx = win_starts[:, None] - oi[None]
+        rel_idx[rel_idx < 0] = 0
+
+        w, o = rel_idx.shape
+        batch_idx = np.arange(b, dtype=np.int32)[:, None, None]
+        batch_idx = np.broadcast_to(batch_idx, (b, w, o))
+        time_idx = np.broadcast_to(rel_idx, (b, w, o))
+
+        return inputs[batch_idx, time_idx]
+
+    output_indices = [800, 74, 671, 132, 818]
+    out = fancy_strided_output(arr, output_indices, stride=16)
+    assert out.shape == (2, 12, 5, 10, 8, 3)
+
+
+dtypes = [np.int32, np.float32, np.float64, np.uint8]
+
+# Shapes for broadcast_to
+broadcast_shapes = [
+    ((10,), (50,), (4,), (3,)),
+    ((8, 6), (16, 12), (4, 3), (1, 3)),
+    ((2, 6), (2, 30), (3, 2), (1, 1)),
+    ((1, 1, 3), (2, 4, 3), (1, 1, 2), (1, 1, 1)),
+]
+
+meshgrid_shapes = [
+    ((10, 20), (3,), (1,)),
+    ((8, 6), (4,), (3,)),
+    ((2, 30), (2,), (1,)),
+    ((20, 4, 3), (4,), (1,)),
+]
+
+
+@pytest.mark.parametrize("dtype", dtypes)
+@pytest.mark.parametrize(("src_shape", "dst_shape", "chunks", "blocks"), broadcast_shapes)
+def test_broadcast_to(dtype, src_shape, dst_shape, chunks, blocks):
+    arr_np = np.arange(np.prod(src_shape), dtype=dtype).reshape(src_shape)
+    arr_b2 = blosc2.asarray(arr_np, chunks=chunks, blocks=blocks)
+
+    try:
+        np_broadcast = np.broadcast_to(arr_np, dst_shape)
+        np_error = None
+    except ValueError as e:
+        np_broadcast = None
+        np_error = e
+
+    if np_error is not None:
+        with pytest.raises(type(np_error)):
+            blosc2.broadcast_to(arr_b2, dst_shape)
+    else:
+        b2_broadcast = blosc2.broadcast_to(arr_b2, dst_shape)
+        assert np.array_equal(b2_broadcast[:], np_broadcast)
+
+
+@pytest.mark.parametrize("dtype", dtypes)
+@pytest.mark.parametrize(("shapes", "chunks", "blocks"), meshgrid_shapes)
+@pytest.mark.parametrize("indexing", ["xy", "ij"])
+def test_meshgrid(dtype, shapes, chunks, blocks, indexing):
+    arrays_np = [np.arange(np.prod(shape), dtype=dtype).reshape(shape) for shape in shapes]
+    arrays_b2 = [blosc2.asarray(a, chunks=chunks, blocks=blocks) for a in arrays_np]
+    try:
+        np_grids = np.meshgrid(*arrays_np, indexing=indexing)
+        np_error = None
+    except ValueError as e:
+        np_grids = None
+        np_error = e
+
+    if np_error is not None:
+        with pytest.raises(type(np_error)):
+            blosc2.meshgrid(*arrays_b2, indexing=indexing)
+    else:
+        b2_grids = blosc2.meshgrid(*arrays_b2, indexing=indexing)
+        assert len(b2_grids) == len(np_grids)
+        for g_b2, g_np in zip(b2_grids, np_grids, strict=False):
+            assert np.array_equal(g_b2[:], g_np)
