@@ -206,6 +206,17 @@ def test_reduce_slice(reduce_op):
     nres = getattr(na[_slice], reduce_op)(axis=1)
     np.testing.assert_allclose(res, nres, atol=tol, rtol=tol)
 
+    # Test reductions with ints
+    _slice = (0, slice(1, 9, 1))
+    res = getattr(a, reduce_op)(item=_slice, axis=1)
+    nres = getattr(na[_slice], reduce_op)(axis=1)
+    np.testing.assert_allclose(res, nres, atol=tol, rtol=tol)
+
+    _slice = (0, slice(1, 9, 2))
+    res = getattr(a, reduce_op)(item=_slice, axis=1)
+    nres = getattr(na[_slice], reduce_op)(axis=1)
+    np.testing.assert_allclose(res, nres, atol=tol, rtol=tol)
+
 
 # Test fast path for reductions
 @pytest.mark.parametrize(
@@ -441,12 +452,18 @@ def test_save_constructor_reduce2(shape, disk, compute):
 def test_reduction_index():
     shape = (20, 20)
     a = blosc2.linspace(0, 20, num=np.prod(shape), shape=shape)
-    arr = blosc2.lazyexpr("sum(a,axis=0)", {"a": a})
+    arr = blosc2.lazyexpr("sum(a, axis=0)", {"a": a})
     newarr = arr.compute()
     assert arr[:10].shape == (10,)
     assert arr[0].shape == ()
     assert arr.shape == newarr.shape
 
+    a = blosc2.ones(shape=(0, 0))
+    with pytest.raises(np.exceptions.AxisError):
+        arr = blosc2.lazyexpr("sum(a, axis=(0, 1, 2))", {"a": a})
+    with pytest.raises(ValueError):
+        arr = blosc2.lazyexpr("sum(a, axis=(0, 0))", {"a": a})
+
 
 @pytest.mark.parametrize("idx", [0, 1, (0,), slice(1, 2), (slice(0, 1),), slice(0, 4), (0, 2)])
 def test_reduction_index2(idx):
@@ -472,15 +489,15 @@ def test_slice_lazy():
 def test_slicebrackets_lazy():
     shape = (20, 20)
     a = blosc2.linspace(0, 20, num=np.prod(shape), shape=shape)
-    arr = blosc2.lazyexpr("anarr[10:15] + 1", {"anarr": a})
+    arr = blosc2.lazyexpr("sum(anarr[10:15], axis=0) + anarr[10:15] + arange(20) + 1", {"anarr": a})
     newarr = arr.compute()
-    np.testing.assert_allclose(newarr[:], a[10:15] + 1)
+    np.testing.assert_allclose(newarr[:], np.sum(a[10:15], axis=0) + a[10:15] + np.arange(20) + 1)
 
     # Try with getitem
     a = blosc2.linspace(0, 20, num=np.prod(shape), shape=shape)
-    arr = blosc2.lazyexpr("anarr[10:15] + 1", {"anarr": a})
+    arr = blosc2.lazyexpr("sum(anarr[10:15], axis=0) + anarr[10:15] + arange(20) + 1", {"anarr": a})
     newarr = arr[:3]
-    res = a[10:15] + 1
+    res = np.sum(a[10:15], axis=0) + a[10:15] + np.arange(20) + 1
     np.testing.assert_allclose(newarr, res[:3])
 
     # Test other cases
@@ -492,6 +509,10 @@ def test_slicebrackets_lazy():
     newarr = arr.compute()
     np.testing.assert_allclose(newarr[:], a[10:15][2:9] + 1)
 
+    arr = blosc2.lazyexpr("sum(anarr[10:15], axis=1) + 1", {"anarr": a})
+    newarr = arr.compute()
+    np.testing.assert_allclose(newarr[:], np.sum(a[10:15], axis=1) + 1)
+
     arr = blosc2.lazyexpr("anarr[10] + 1", {"anarr": a})
     newarr = arr.compute()
     np.testing.assert_allclose(newarr[:], a[10] + 1)
